## $Id: test_catalog.py 12095 2014-11-30 09:12:44Z henrik $
##
## Copyright (C) 2014 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
import os
import grok
import shutil
import tempfile
from zope.event import notify
from zope.catalog.interfaces import ICatalog
from zope.component import queryUtility, createObject
from zope.component.hooks import setSite
from waeup.ikoba.app import Company
from waeup.ikoba.testing import FunctionalLayer, FunctionalTestCase
from waeup.ikoba.customers.customer import Customer
from waeup.ikoba.customers.applications import SampleApplication


class CatalogTestSetup(FunctionalTestCase):
    # A setup for testing catalog related stuff.
    #
    # sets up a site with some customer already created.
    layer = FunctionalLayer

    def setUp(self):
        super(CatalogTestSetup, self).setUp()

        # Prepopulate ZODB
        app = Company()
        self.dc_root = tempfile.mkdtemp()
        app['datacenter'].setStoragePath(self.dc_root)

        # Prepopulate the ZODB...
        self.getRootFolder()['app'] = app
        self.app = self.getRootFolder()['app']
        setSite(self.app)

        # Create customer with subobjects
        customer = Customer()
        customer.firstname = u'Bob'
        customer.lastname = u'Tester'
        customer.matric_number = u'1234'
        self.app['customers'].addCustomer(customer)
        self.customer_id = customer.customer_id
        self.customer = self.app['customers'][self.customer_id]

        product = createObject('waeup.Product')
        product.product_id = u'SAM'
        product.title = u'Our Samle Product'
        product.application_category = u'sample'
        self.app['products'].addProduct(product)
        self.product_id = product.product_id
        self.product = self.app['products'][self.product_id]

        application = createObject('waeup.SampleApplication')
        application.title = u'My Samle Application'
        application.product = product
        application.last_product_id = product.product_id
        self.customer['applications'].addApplication(application)
        self.application_id = application.application_id
        self.application = self.customer['applications'][self.application_id]

        document = createObject('waeup.CustomerSampleDocument')
        document.title = u'My Samle Document'
        self.customer['documents'].addDocument(document)
        self.document_id = document.document_id
        self.document = self.customer['documents'][self.document_id]

        return

    def tearDown(self):
        shutil.rmtree(self.dc_root)
        super(CatalogTestSetup, self).tearDown()
        return


class CustomersCatalogTests(CatalogTestSetup):

    layer = FunctionalLayer

    def test_get_catalog(self):
        # We can get an customers catalog if we wish
        cat = queryUtility(ICatalog, name='customers_catalog')
        assert cat is not None

    def test_search_by_id(self):
        # We can find a certain customer id
        cat = queryUtility(ICatalog, name='customers_catalog')
        results = cat.searchResults(customer_id=(self.customer_id,
                                                self.customer_id))
        results = [x for x in results] # Turn results generator into list
        assert len(results) == 1
        assert results[0] is self.app['customers'][self.customer_id]

    def test_search_by_name(self):
        # We can find a certain name
        cat = queryUtility(ICatalog, name='customers_catalog')
        results = cat.searchResults(fullname='Bob Tester')
        results = [x for x in results] # Turn results generator into list
        assert len(results) == 1
        assert results[0] is self.app['customers'][self.customer_id]


class DocumentsCatalogTests(CatalogTestSetup):

    layer = FunctionalLayer

    def test_get_catalog(self):
        # We can get an customers catalog if we wish
        cat = queryUtility(ICatalog, name='documents_catalog')
        assert cat is not None

    def test_search_by_id(self):
        # We can find a certain application id
        cat = queryUtility(ICatalog, name='documents_catalog')
        results = cat.searchResults(document_id=(self.document_id,
                                                self.document_id))
        results = [x for x in results] # Turn results generator into list
        assert len(results) == 1
        assert results[0] is self.customer['documents'][self.document_id]


class ApplicationsCatalogTests(CatalogTestSetup):

    layer = FunctionalLayer

    def test_get_catalog(self):
        # We can get an customers catalog if we wish
        cat = queryUtility(ICatalog, name='applications_catalog')
        assert cat is not None

    def test_search_by_id(self):
        # We can find a certain application id
        cat = queryUtility(ICatalog, name='applications_catalog')
        results = cat.searchResults(application_id=(self.application_id,
                                                self.application_id))
        results = [x for x in results] # Turn results generator into list
        assert len(results) == 1
        assert results[0] is self.customer['applications'][self.application_id]

    def test_search_by_category(self):
        # We can find a certain name
        cat = queryUtility(ICatalog, name='applications_catalog')
        results = cat.searchResults(application_category=('sample', 'sample'))
        results = [x for x in results] # Turn results generator into list
        assert len(results) == 1
        assert results[0] is self.customer['applications'][self.application_id]

    def test_search_by_last_product_id(self):
        # We can find a certain name
        cat = queryUtility(ICatalog, name='applications_catalog')
        results = cat.searchResults(last_product_id=('SAM', 'SAM'))
        results = [x for x in results] # Turn results generator into list
        assert len(results) == 1
        assert results[0] is self.customer['applications'][self.application_id]
        assert results[0].product is self.product

    def test_product_removal(self):
        # This test does not only test catalog components, it also ensures
        # that the product attribute of an application is cleared
        # but the last_product_id is not.
        del self.app['products'][self.product_id]
        self.assertTrue(self.application.product is None)
        self.assertEqual(self.application.last_product_id, 'SAM')
        cat = queryUtility(ICatalog, name='applications_catalog')
        results = cat.searchResults(last_product_id=('SAM', 'SAM'))
        results = [x for x in results]
        assert len(results) == 1
        assert results[0] is self.customer['applications'][self.application_id]
        assert results[0].product is None
        # Product removal is being logged in customers.log
        logfile = os.path.join(
            self.app['datacenter'].storage, 'logs', 'customers.log')
        logcontent = open(logfile).read()
        self.assertTrue(
            'INFO - system - ObjectRemovedEvent - K1000000 - a101 - removed: product\n'
            in logcontent)

